@@ -4,8 +4,6 @@
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
 #undef HAVE_STRUCT_OSOCKADDR
-#undef PACKAGE
-#undef VERSION
 #undef HAVE_LIBSM
 #undef HAVE_LIBINTL_H
 #undef HAVE_GTK_SIGNAL_HANDLER_PENDING
@@ -25,4 +23,13 @@
 #undef USE_NETCODE
 #undef USE_TERMNET
 #undef USE_VFS
-#undef HAVE_ORBIT
+#undef HAVE_ESD
+#undef PACKAGE
+#undef VERSION
+#undef USE_POPT
+#undef HAVE_LIBAUDIOFILE
+#undef umode_t
+#undef HAVE_OPENPTY
+#undef HAVE_PTY_H
+#undef HAVE_UTIL_H
+#undef HAVE_SENDMSG
