@@ -1,14 +1,14 @@
 #ifndef lint
-static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/readPNG.c,v 1.7 1998/06/29 06:32:14 tromey Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/readPNG.c,v 1.8 1998/09/25 01:09:12 unammx Exp $";
 #endif
 /*****
 * readPNG.c : XmHTML png image loading routines
 *
-* This file Version	$Revision: 1.7 $
+* This file Version	$Revision: 1.8 $
 *
 * Creation date:		Wed Feb 19 03:21:11 GMT+0100 1997
-* Last modification: 	$Date: 1998/06/29 06:32:14 $
-* By:					$Author: tromey $
+* Last modification: 	$Date: 1998/09/25 01:09:12 $
+* By:					$Author: unammx $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -37,6 +37,12 @@ static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/readPNG.c,v 1.7 1
 /*****
 * ChangeLog 
 * $Log: readPNG.c,v $
+* Revision 1.8  1998/09/25 01:09:12  unammx
+* 1998-09-24  Miguel de Icaza  <miguel@nuclecu.unam.mx>
+*
+* 	* readPNG.c (_XmHTMLReadPNG): I am the one to blame.  Fix PNG file
+* 	loading.  I do not know how I ever commited this.
+*
 * Revision 1.7  1998/06/29 06:32:14  tromey
 * added gnome-config to .cvsignore
 *
