@@ -7,6 +7,13 @@
  * Adatped to the GNOME needs by:
  *    Elliot Lee (sopwith@cuc.edu)
  */
+
+/* needed for S_ISSOCK with 'gcc -ansi -pedantic' on GNU/Linux */
+#ifndef _BSD_SOURCE
+#  define _BSD_SOURCE 1
+#endif
+#include <sys/types.h>
+
 #include <libgnome/libgnome.h>
 #include <ctype.h>
 #include <unistd.h>
