@@ -6,6 +6,13 @@
  * Author: Miguel de Icaza, 
  */
 #include <config.h>
+
+/* needed for S_ISLNK with 'gcc -ansi -pedantic' on GNU/Linux */
+#ifndef _BSD_SOURCE
+#  define _BSD_SOURCE 1
+#endif
+#include <sys/types.h>
+
 #include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -31,7 +38,7 @@ gnome_dirrelative_file (const char *base, const char *sub, const char *filename,
 	
 	if (gnomedir) {
 		t = g_concat_dir_and_file (gnomedir, sub);
-		u = g_copy_strings (t, "/", filename, NULL);
+		u = g_strconcat (t, "/", filename, NULL);
 		g_free (t); t = NULL;
 		
 		if (g_file_exists (u)) {
@@ -39,7 +46,7 @@ gnome_dirrelative_file (const char *base, const char *sub, const char *filename,
 		}
 
 		t = g_concat_dir_and_file (gnome_util_user_home (), sub);
-		v = g_copy_strings (t, "/", filename, NULL);
+		v = g_strconcat (t, "/", filename, NULL);
 		g_free (t); t = NULL;
 
 		if (g_file_exists (v)){
@@ -269,7 +276,10 @@ g_file_exists (const char *filename)
  *
  * returns a new allocated char * with the concatenation of its arguments,
  * the list of strings is terminated by a NULL pointer.
+ *
+ * NOTE: This function is deprecated.  Use GLib's g_strconcat() instead.
  */
+#undef g_copy_strings
 char *
 g_copy_strings (const char *first, ...)
 {
@@ -298,6 +308,7 @@ g_copy_strings (const char *first, ...)
 	
 	return result;
 }
+#define g_copy_strings g_strconcat
 
 /**
  * g_unix_error_string:
@@ -332,9 +343,9 @@ g_concat_dir_and_file (const char *dir, const char *file)
         /* If the directory name doesn't have a / on the end, we need
 	   to add one so we get a proper path to the file */
 	if (dir [strlen(dir) - 1] != PATH_SEP)
-		return g_copy_strings (dir, PATH_SEP_STR, file, NULL);
+		return g_strconcat (dir, PATH_SEP_STR, file, NULL);
 	else
-		return g_copy_strings (dir, file, NULL);
+		return g_strconcat (dir, file, NULL);
 }
 
 /**
