@@ -91,6 +91,14 @@ gnome_app_get_type (void)
 	return gnomeapp_type;
 }
 
+static void
+gnome_app_add (GtkContainer *container, GtkWidget *widget)
+{
+	GnomeApp *gnome_app = GNOME_APP (container);
+
+	gnome_app_set_contents (gnome_app, widget);
+}
+
 static void
 gnome_app_class_init (GnomeAppClass *class)
 {
@@ -225,7 +233,7 @@ gnome_app_construct (GnomeApp *app, gchar *appname, char *title)
 	g_return_if_fail (appname != NULL);
 
 	app->name = g_strdup (appname);
-	app->prefix = g_copy_strings ("/", appname, "/", NULL);
+	app->prefix = g_strconcat ("/", appname, "/", NULL);
 	
 	if (title)
 		gtk_window_set_title (GTK_WINDOW (app), title);
