@@ -4,6 +4,14 @@
  */
 
 #include <config.h>
+
+/* needed for values of M_E and M_PI under 'gcc -ansi -pedantic'
+ * on GNU/Linux */
+#ifndef _BSD_SOURCE
+#  define _BSD_SOURCE 1
+#endif
+#include <sys/types.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
@@ -23,7 +31,7 @@ typedef void (*sighandler_t)(int);
 typedef enum {
 	CALCULATOR_NUMBER,
 	CALCULATOR_FUNCTION,
-	CALCULATOR_PARENTHESIS,
+	CALCULATOR_PARENTHESIS
 } CalculatorActionType;
 
 typedef gdouble (*MathFunction1) (gdouble);
@@ -1030,9 +1038,11 @@ sub_parenth(GtkWidget *w, gpointer data)
 			stack_pop(&gc->stack);
 		else if(g_list_next(gc->stack)) {
 			stack = g_list_next(gc->stack)->data;
-			if(stack->type==CALCULATOR_PARENTHESIS)
-				gc->stack = g_list_remove_link(gc->stack,
-						       g_list_next(gc->stack));
+			if(stack->type==CALCULATOR_PARENTHESIS) {
+				GList *n = g_list_next(gc->stack);
+				gc->stack = g_list_remove_link(gc->stack,n);
+				g_list_free_1(n);
+			}
 		}
 	}
 
