@@ -1,4 +1,4 @@
-/* Rectangle and ellipse item types for GnomeCanvas widgetr
+/* Rectangle and ellipse item types for GnomeCanvas widget
  *
  * GnomeCanvas is basically a port of the Tk toolkit's most excellent canvas widget.  Tk is
  * copyrighted by the Regents of the University of California, Sun Microsystems, and other parties.
@@ -20,6 +20,7 @@
 
 /* Base class for rectangle and ellipse item types */
 
+#define noVERBOSE
 
 enum {
 	ARG_0,
@@ -29,8 +30,10 @@ enum {
 	ARG_Y2,
 	ARG_FILL_COLOR,
 	ARG_FILL_COLOR_GDK,
+	ARG_FILL_COLOR_RGBA,
 	ARG_OUTLINE_COLOR,
 	ARG_OUTLINE_COLOR_GDK,
+	ARG_OUTLINE_COLOR_RGBA,
 	ARG_FILL_STIPPLE,
 	ARG_OUTLINE_STIPPLE,
 	ARG_WIDTH_PIXELS,
@@ -102,8 +105,10 @@ gnome_canvas_re_class_init (GnomeCanvasREClass *class)
 	gtk_object_add_arg_type ("GnomeCanvasRE::y2", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_Y2);
 	gtk_object_add_arg_type ("GnomeCanvasRE::fill_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_FILL_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasRE::fill_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_FILL_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasRE::fill_color_rgba", GTK_TYPE_UINT, GTK_ARG_READWRITE, ARG_FILL_COLOR_RGBA);
 	gtk_object_add_arg_type ("GnomeCanvasRE::outline_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_OUTLINE_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasRE::outline_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_OUTLINE_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasRE::outline_color_rgba", GTK_TYPE_UINT, GTK_ARG_READWRITE, ARG_OUTLINE_COLOR_RGBA);
 	gtk_object_add_arg_type ("GnomeCanvasRE::fill_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_FILL_STIPPLE);
 	gtk_object_add_arg_type ("GnomeCanvasRE::outline_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_OUTLINE_STIPPLE);
 	gtk_object_add_arg_type ("GnomeCanvasRE::width_pixels", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_WIDTH_PIXELS);
@@ -149,18 +154,26 @@ gnome_canvas_re_destroy (GtkObject *object)
 	if (re->outline_stipple)
 		gdk_bitmap_unref (re->outline_stipple);
 
+	if (re->fill_svp)
+		art_svp_free (re->fill_svp);
+
+	if (re->outline_svp)
+		art_svp_free (re->outline_svp);
+
 	if (GTK_OBJECT_CLASS (re_parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (re_parent_class)->destroy) (object);
 }
 
-static void
-recalc_bounds (GnomeCanvasRE *re)
+static void get_bounds (GnomeCanvasRE *re, double *px1, double *py1, double *px2, double *py2)
 {
 	GnomeCanvasItem *item;
 	double x1, y1, x2, y2;
 	int cx1, cy1, cx2, cy2;
 	double hwidth;
 
+#ifdef VERBOSE
+	g_print ("re get_bounds\n");
+#endif
 	item = GNOME_CANVAS_ITEM (re);
 
 	if (re->width_pixels)
@@ -177,22 +190,36 @@ recalc_bounds (GnomeCanvasRE *re)
 	gnome_canvas_item_i2w (item, &x2, &y2);
 	gnome_canvas_w2c (item->canvas, x1 - hwidth, y1 - hwidth, &cx1, &cy1);
 	gnome_canvas_w2c (item->canvas, x2 + hwidth, y2 + hwidth, &cx2, &cy2);
-	item->x1 = cx1;
-	item->y1 = cy1;
-	item->x2 = cx2;
-	item->y2 = cy2;
+	*px1 = cx1;
+	*py1 = cy1;
+	*px2 = cx2;
+	*py2 = cy2;
 
 	/* Some safety fudging */
 
-	item->x1 -= 2;
-	item->y1 -= 2;
-	item->x2 += 2;
-	item->y2 += 2;
+	*px1 -= 2;
+	*py1 -= 2;
+	*px2 += 2;
+	*py2 += 2;
+}
+
+/* deprecated */
+static void
+recalc_bounds (GnomeCanvasRE *re)
+{
+	GnomeCanvasItem *item;
+	double x1, y1, x2, y2;
+	int cx1, cy1, cx2, cy2;
+	double hwidth;
+
+#ifdef VERBOSE
+	g_print ("re recalc_bounds\n");
+#endif
+	item = GNOME_CANVAS_ITEM (re);
+
+	get_bounds (re, &item->x1, &item->y1, &item->x2, &item->y2);
 
 	gnome_canvas_group_child_bounds (GNOME_CANVAS_GROUP (item->parent), item);
-
-	if (item->canvas->aa)
-		gnome_canvas_item_request_update (item);
 }
 
 /* Convenience function to set a GC's foreground color to the specified pixel value */
@@ -246,6 +273,24 @@ set_outline_gc_width (GnomeCanvasRE *re)
 }
 
 
+static void
+gnome_canvas_re_set_fill (GnomeCanvasRE *re, gboolean fill_set)
+{
+	if (re->fill_set != fill_set) {
+		re->fill_set = fill_set;
+		gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (re));
+	}
+}
+
+static void
+gnome_canvas_re_set_outline (GnomeCanvasRE *re, gboolean outline_set)
+{
+	if (re->outline_set != outline_set) {
+		re->outline_set = outline_set;
+		gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (re));
+	}
+}
+
 static void
 gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
@@ -259,55 +304,85 @@ gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	switch (arg_id) {
 	case ARG_X1:
 		re->x1 = GTK_VALUE_DOUBLE (*arg);
-		recalc_bounds (re);
+
+		gnome_canvas_item_request_update (item);
 		break;
 
 	case ARG_Y1:
 		re->y1 = GTK_VALUE_DOUBLE (*arg);
-		recalc_bounds (re);
+
+		gnome_canvas_item_request_update (item);
 		break;
 
 	case ARG_X2:
 		re->x2 = GTK_VALUE_DOUBLE (*arg);
-		recalc_bounds (re);
+
+		gnome_canvas_item_request_update (item);
 		break;
 
 	case ARG_Y2:
 		re->y2 = GTK_VALUE_DOUBLE (*arg);
-		recalc_bounds (re);
+
+		gnome_canvas_item_request_update (item);
 		break;
 
 	case ARG_FILL_COLOR:
 		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color)) {
-			re->fill_set = TRUE;
+			gnome_canvas_re_set_fill (re, TRUE);
 			re->fill_pixel = color.pixel;
-			if (item->canvas->aa)
+			if (item->canvas->aa) {
 				re->fill_color =
 					((color.red & 0xff00) << 16) |
 					((color.green & 0xff00) << 8) |
 					(color.blue & 0xff00) |
 					0xff;
+#ifdef VERBOSE			       
+				g_print ("re fill color = %08x\n", re->fill_color);
+#endif
+			}
 			else
 				set_gc_foreground (re->fill_gc, re->fill_pixel);
 		} else {
-			re->fill_set = FALSE;
+			gnome_canvas_re_set_fill (re, FALSE);
 			re->fill_color = 0;
 		}
 
-		if (item->canvas->aa)
-			gnome_canvas_item_request_update (item);
+		gnome_canvas_item_request_redraw_svp (item, re->fill_svp);
 
 		break;
 
 	case ARG_FILL_COLOR_GDK:
-		re->fill_set = TRUE;
+		gnome_canvas_re_set_fill (re, TRUE);
 		re->fill_pixel = ((GdkColor *) GTK_VALUE_BOXED (*arg))->pixel;
-		set_gc_foreground (re->fill_gc, re->fill_pixel);
+
+		if (item->canvas->aa)
+			re->fill_color =
+				((color.red & 0xff00) << 16) |
+				((color.green & 0xff00) << 8) |
+				(color.blue & 0xff00) |
+				0xff;
+		else
+			set_gc_foreground (re->fill_gc, re->fill_pixel);
+
+		gnome_canvas_item_request_redraw_svp (item, re->fill_svp);
+
+		break;
+
+	case ARG_FILL_COLOR_RGBA:
+		gnome_canvas_re_set_fill (re, TRUE);
+		re->fill_set = TRUE;
+		re->fill_color = GTK_VALUE_UINT (*arg);
+
+		gnome_canvas_item_request_redraw_svp (item, re->fill_svp);
+
 		break;
 
 	case ARG_OUTLINE_COLOR:
 		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color)) {
-			re->outline_set = TRUE;
+#ifdef VERBOSE
+			g_print ("re outline color %x %x %x\n", color.red, color.green, color.blue);
+#endif
+			gnome_canvas_re_set_outline (re, TRUE);
 			re->outline_pixel = color.pixel;
 			if (item->canvas->aa)
 				re->outline_color =
@@ -318,38 +393,68 @@ gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 			else
 				set_gc_foreground (re->outline_gc, re->outline_pixel);
 		} else {
-			re->outline_set = FALSE;
+			gnome_canvas_re_set_outline (re, FALSE);
 			re->outline_color = 0;
 		}
 
+		gnome_canvas_item_request_redraw_svp (item, re->outline_svp);
+
 		break;
 
 	case ARG_OUTLINE_COLOR_GDK:
+		gnome_canvas_re_set_outline (re, TRUE);
 		re->outline_set = TRUE;
 		re->outline_pixel = ((GdkColor *) GTK_VALUE_BOXED (*arg))->pixel;
-		set_gc_foreground (re->outline_gc, re->outline_pixel);
+
+		if (item->canvas->aa)
+			re->outline_color =
+				((color.red & 0xff00) << 16) |
+				((color.green & 0xff00) << 8) |
+				(color.blue & 0xff00) |
+				0xff;
+		else
+			set_gc_foreground (re->outline_gc, re->outline_pixel);
+
+		gnome_canvas_item_request_redraw_svp (item, re->outline_svp);
+
+		break;
+
+	case ARG_OUTLINE_COLOR_RGBA:
+		gnome_canvas_re_set_outline (re, TRUE);
+		re->outline_color = GTK_VALUE_UINT (*arg);
+
+		gnome_canvas_item_request_redraw_svp (item, re->outline_svp);
+
 		break;
 
 	case ARG_FILL_STIPPLE:
-		set_stipple (re->fill_gc, &re->fill_stipple, GTK_VALUE_BOXED (*arg), FALSE);
+		if (!item->canvas->aa)
+			set_stipple (re->fill_gc, &re->fill_stipple, GTK_VALUE_BOXED (*arg), FALSE);
 		break;
 
 	case ARG_OUTLINE_STIPPLE:
-		set_stipple (re->outline_gc, &re->outline_stipple, GTK_VALUE_BOXED (*arg), FALSE);
+		if (!item->canvas->aa)
+			set_stipple (re->outline_gc, &re->outline_stipple, GTK_VALUE_BOXED (*arg), FALSE);
 		break;
 		
 	case ARG_WIDTH_PIXELS:
 		re->width = GTK_VALUE_UINT (*arg);
 		re->width_pixels = TRUE;
-		set_outline_gc_width (re);
-		recalc_bounds (re);
+		if (!item->canvas->aa)
+			set_outline_gc_width (re);
+
+		gnome_canvas_item_request_update (item);
+	
 		break;
 
 	case ARG_WIDTH_UNITS:
 		re->width = fabs (GTK_VALUE_DOUBLE (*arg));
 		re->width_pixels = FALSE;
-		set_outline_gc_width (re);
-		recalc_bounds (re);
+		if (!item->canvas->aa)
+			set_outline_gc_width (re);
+
+		gnome_canvas_item_request_update (item);
+
 		break;
 
 	default:
@@ -403,6 +508,14 @@ gnome_canvas_re_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		get_color_arg (re, re->outline_pixel, arg);
 		break;
 
+	case ARG_FILL_COLOR_RGBA:
+		GTK_VALUE_UINT (*arg) = re->fill_color;
+		break;
+
+	case ARG_OUTLINE_COLOR_RGBA:
+		GTK_VALUE_UINT (*arg) = re->outline_color;
+		break;
+
 	case ARG_FILL_STIPPLE:
 		GTK_VALUE_BOXED (*arg) = re->fill_stipple;
 		break;
@@ -422,6 +535,9 @@ gnome_canvas_re_update_shared (GnomeCanvasItem *item, double *affine, ArtSVP *cl
 {
 	GnomeCanvasRE *re;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_re_update_shared\n");
+#endif
 	re = GNOME_CANVAS_RE (item);
 
 	if (re_parent_class->update)
@@ -435,7 +551,9 @@ gnome_canvas_re_update_shared (GnomeCanvasItem *item, double *affine, ArtSVP *cl
 		set_outline_gc_width (re);
 	} 
 
+#ifdef OLD_XFORM
 	recalc_bounds (re);
+#endif
 }
 
 static void
@@ -443,15 +561,22 @@ gnome_canvas_re_realize (GnomeCanvasItem *item)
 {
 	GnomeCanvasRE *re;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_re_realize\n");
+#endif
 	re = GNOME_CANVAS_RE (item);
 
 	if (re_parent_class->realize)
 		(* re_parent_class->realize) (item);
 
-	re->fill_gc = gdk_gc_new (item->canvas->layout.bin_window);
-	re->outline_gc = gdk_gc_new (item->canvas->layout.bin_window);
+	if (!item->canvas->aa) {
+		re->fill_gc = gdk_gc_new (item->canvas->layout.bin_window);
+		re->outline_gc = gdk_gc_new (item->canvas->layout.bin_window);
+	}
 
+#ifdef OLD_XFORM
 	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
+#endif
 }
 
 static void
@@ -461,8 +586,10 @@ gnome_canvas_re_unrealize (GnomeCanvasItem *item)
 
 	re = GNOME_CANVAS_RE (item);
 
-	gdk_gc_unref (re->fill_gc);
-	gdk_gc_unref (re->outline_gc);
+	if (!item->canvas->aa) {
+		gdk_gc_unref (re->fill_gc);
+		gdk_gc_unref (re->outline_gc);
+	}
 
 	if (re_parent_class->unrealize)
 		(* re_parent_class->unrealize) (item);
@@ -473,6 +600,9 @@ gnome_canvas_re_translate (GnomeCanvasItem *item, double dx, double dy)
 {
 	GnomeCanvasRE *re;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_re_translate %g %g\n", dx, dy);
+#endif
 	re = GNOME_CANVAS_RE (item);
 
 	re->x1 += dx;
@@ -481,6 +611,13 @@ gnome_canvas_re_translate (GnomeCanvasItem *item, double dx, double dy)
 	re->y2 += dy;
 
 	recalc_bounds (re);
+
+#ifdef VERBOSE
+	g_print ("translate\n");
+#endif
+	if (item->canvas->aa) {
+		gnome_canvas_item_request_update (item);
+	}
 }
 
 static void
@@ -489,6 +626,9 @@ gnome_canvas_re_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x
 	GnomeCanvasRE *re;
 	double hwidth;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_re_bounds\n");
+#endif
 	re = GNOME_CANVAS_RE (item);
 
 	if (re->width_pixels)
@@ -511,6 +651,11 @@ gnome_canvas_re_render (GnomeCanvasItem *item,
 
 	re = GNOME_CANVAS_RE (item);
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_re_render (%d, %d) - (%d, %d) fill=%08x outline=%08x\n",
+		 buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1, re->fill_color, re->outline_color);
+#endif
+
 	if (re->fill_svp != NULL) {
 		gnome_canvas_render_svp (buf, re->fill_svp, re->fill_color);
 	}
@@ -569,21 +714,30 @@ static void
 gnome_canvas_rect_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height)
 {
 	GnomeCanvasRE *re;
-	double dx, dy;
+	double i2w[6], w2c[6], i2c[6];
 	int x1, y1, x2, y2;
+	ArtPoint i1, i2;
+	ArtPoint c1, c2;
 
 	re = GNOME_CANVAS_RE (item);
 
 	/* Get canvas pixel coordinates */
 
-	dx = 0.0;
-	dy = 0.0;
-
-	gnome_canvas_item_i2w (item, &dx, &dy);
-
-	gnome_canvas_w2c (item->canvas, dx + re->x1, dy + re->y1, &x1, &y1);
-	gnome_canvas_w2c (item->canvas, dx + re->x2, dy + re->y2, &x2, &y2);
+	gnome_canvas_item_i2w_affine (item, i2w);
+	gnome_canvas_w2c_affine (item->canvas, w2c);
+	art_affine_multiply (i2c, i2w, w2c);
 
+	i1.x = re->x1;
+	i1.y = re->y1;
+	i2.x = re->x2;
+	i2.y = re->y2;
+	art_affine_point (&c1, &i1, i2c);
+	art_affine_point (&c2, &i2, i2c);
+	x1 = c1.x;
+	y1 = c1.y;
+	x2 = c2.x;
+	y2 = c2.y;
+	
 	if (re->fill_set) {
 		if (re->fill_stipple)
 			gnome_canvas_set_stipple_origin (item->canvas, re->fill_gc);
@@ -620,6 +774,9 @@ gnome_canvas_rect_point (GnomeCanvasItem *item, double x, double y, int cx, int
 	double dx, dy;
 	double tmp;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_rect_point\n");
+#endif
 	re = GNOME_CANVAS_RE (item);
 
 	*actual_item = item;
@@ -695,114 +852,135 @@ gnome_canvas_rect_update (GnomeCanvasItem *item, double affine[6], ArtSVP *clip_
 {
 	GnomeCanvasRE *re;
 	ArtVpath vpath[11];
+	ArtVpath *vpath2;
 	double x0, y0, x1, y1;
 	double dx, dy;
 	double halfwidth;
 	int i;
 
+#ifdef VERBOSE
+	{
+		char str[128];
+
+		art_affine_to_string (str, affine);
+		g_print ("gnome_canvas_rect_update item %x %s\n", item, str);
+
+		if (item->xform) {
+		  g_print ("xform = %g %g\n", item->xform[0], item->xform[1]);
+		}
+	}
+#endif
 	gnome_canvas_re_update_shared (item, affine, clip_path, flags);
 
 	re = GNOME_CANVAS_RE (item);
 
-	x0 = re->x1 * item->canvas->pixels_per_unit;
-	y0 = re->y1 * item->canvas->pixels_per_unit;
-	x1 = re->x2 * item->canvas->pixels_per_unit;
-	y1 = re->y2 * item->canvas->pixels_per_unit;
+	if (item->canvas->aa) {
+		x0 = re->x1;
+		y0 = re->y1;
+		x1 = re->x2;
+		y1 = re->y2;
 
-	if (re->fill_svp)
-		art_svp_free (re->fill_svp);
+		gnome_canvas_item_reset_bounds (item);
 
-	if (re->fill_set) {
-		vpath[0].code = ART_MOVETO;
-		vpath[0].x = x0;
-		vpath[0].y = y0;
-		vpath[1].code = ART_LINETO;
-		vpath[1].x = x0;
-		vpath[1].y = y1;
-		vpath[2].code = ART_LINETO;
-		vpath[2].x = x1;
-		vpath[2].y = y1;
-		vpath[3].code = ART_LINETO;
-		vpath[3].x = x1;
-		vpath[3].y = y0;
-		vpath[4].code = ART_LINETO;
-		vpath[4].x = x0;
-		vpath[4].y = y0;
-		vpath[5].code = ART_END;
-		vpath[5].x = 0;
-		vpath[5].y = 0;
+		if (re->fill_set) {
+			vpath[0].code = ART_MOVETO;
+			vpath[0].x = x0;
+			vpath[0].y = y0;
+			vpath[1].code = ART_LINETO;
+			vpath[1].x = x0;
+			vpath[1].y = y1;
+			vpath[2].code = ART_LINETO;
+			vpath[2].x = x1;
+			vpath[2].y = y1;
+			vpath[3].code = ART_LINETO;
+			vpath[3].x = x1;
+			vpath[3].y = y0;
+			vpath[4].code = ART_LINETO;
+			vpath[4].x = x0;
+			vpath[4].y = y0;
+			vpath[5].code = ART_END;
+			vpath[5].x = 0;
+			vpath[5].y = 0;
 
-		re->fill_svp = art_svp_from_vpath (vpath);
-	} else
-		re->fill_svp = NULL;
+			vpath2 = art_vpath_affine_transform (vpath, affine);
 
-	if (re->outline_svp)
-		art_svp_free (re->outline_svp);
+			gnome_canvas_item_update_svp_clip (item, &re->fill_svp, art_svp_from_vpath (vpath2), clip_path);
+			art_free (vpath2);
+		} else
+			gnome_canvas_item_update_svp (item, &re->fill_svp, NULL);
 
-	if (re->outline_set) {
-		/* We do the stroking by hand because it's simple enough
-		   and could save time. */
+		if (re->outline_set) {
+			/* We do the stroking by hand because it's simple enough
+			   and could save time. */
 
-		if (re->width_pixels)
-			halfwidth = re->width * 0.5;
-		else
-			halfwidth = re->width * item->canvas->pixels_per_unit * 0.5;
+			if (re->width_pixels)
+				halfwidth = re->width * 0.5;
+			else
+				halfwidth = re->width * item->canvas->pixels_per_unit * 0.5;
 
-		i = 0;
-		vpath[i].code = ART_MOVETO;
-		vpath[i].x = x0 - halfwidth;
-		vpath[i].y = y0 - halfwidth;
-		i++;
-		vpath[i].code = ART_LINETO;
-		vpath[i].x = x0 - halfwidth;
-		vpath[i].y = y1 + halfwidth;
-		i++;
-		vpath[i].code = ART_LINETO;
-		vpath[i].x = x1 + halfwidth;
-		vpath[i].y = y1 + halfwidth;
-		i++;
-		vpath[i].code = ART_LINETO;
-		vpath[i].x = x1 + halfwidth;
-		vpath[i].y = y0 - halfwidth;
-		i++;
-		vpath[i].code = ART_LINETO;
-		vpath[i].x = x0 - halfwidth;
-		vpath[i].y = y0 - halfwidth;
-		i++;
+			if (halfwidth < 0.25)
+				halfwidth = 0.25;
 
-		if (x1 - halfwidth > x0 + halfwidth &&
-		    y1 - halfwidth > y0 + halfwidth) {
+			i = 0;
 			vpath[i].code = ART_MOVETO;
-			vpath[i].x = x0 + halfwidth;
-			vpath[i].y = y0 + halfwidth;
+			vpath[i].x = x0 - halfwidth;
+			vpath[i].y = y0 - halfwidth;
 			i++;
 			vpath[i].code = ART_LINETO;
-			vpath[i].x = x1 - halfwidth;
-			vpath[i].y = y0 + halfwidth;
+			vpath[i].x = x0 - halfwidth;
+			vpath[i].y = y1 + halfwidth;
 			i++;
 			vpath[i].code = ART_LINETO;
-			vpath[i].x = x1 - halfwidth;
-			vpath[i].y = y1 - halfwidth;
+			vpath[i].x = x1 + halfwidth;
+			vpath[i].y = y1 + halfwidth;
 			i++;
 			vpath[i].code = ART_LINETO;
-			vpath[i].x = x0 + halfwidth;
-			vpath[i].y = y1 - halfwidth;
+			vpath[i].x = x1 + halfwidth;
+			vpath[i].y = y0 - halfwidth;
 			i++;
 			vpath[i].code = ART_LINETO;
-			vpath[i].x = x0 + halfwidth;
-			vpath[i].y = y0 + halfwidth;
+			vpath[i].x = x0 - halfwidth;
+			vpath[i].y = y0 - halfwidth;
 			i++;
-		}
-		vpath[i].code = ART_END;
-		vpath[i].x = 0;
-		vpath[i].y = 0;
 
-		re->outline_svp = art_svp_from_vpath (vpath);
-	} else
-		re->outline_svp = NULL;
+			if (x1 - halfwidth > x0 + halfwidth &&
+			    y1 - halfwidth > y0 + halfwidth) {
+				vpath[i].code = ART_MOVETO;
+				vpath[i].x = x0 + halfwidth;
+				vpath[i].y = y0 + halfwidth;
+				i++;
+				vpath[i].code = ART_LINETO;
+				vpath[i].x = x1 - halfwidth;
+				vpath[i].y = y0 + halfwidth;
+				i++;
+				vpath[i].code = ART_LINETO;
+				vpath[i].x = x1 - halfwidth;
+				vpath[i].y = y1 - halfwidth;
+				i++;
+				vpath[i].code = ART_LINETO;
+				vpath[i].x = x0 + halfwidth;
+				vpath[i].y = y1 - halfwidth;
+				i++;
+				vpath[i].code = ART_LINETO;
+				vpath[i].x = x0 + halfwidth;
+				vpath[i].y = y0 + halfwidth;
+				i++;
+			}
+			vpath[i].code = ART_END;
+			vpath[i].x = 0;
+			vpath[i].y = 0;
 
+			vpath2 = art_vpath_affine_transform (vpath, affine);
 
-	/* FIXME: request repaint */
+			gnome_canvas_item_update_svp_clip (item, &re->outline_svp, art_svp_from_vpath (vpath2), clip_path);
+			art_free (vpath2);
+		} else
+			gnome_canvas_item_update_svp (item, &re->outline_svp, NULL);
+	} else {
+		/* xlib rendering - just update the bbox */
+		get_bounds (re, &x0, &y0, &x1, &y1);
+		gnome_canvas_update_bbox (item, x0, y0, x1, y1);
+	}
 }
 
 /* Ellipse item */
@@ -855,21 +1033,30 @@ static void
 gnome_canvas_ellipse_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height)
 {
 	GnomeCanvasRE *re;
-	double dx, dy;
+	double i2w[6], w2c[6], i2c[6];
 	int x1, y1, x2, y2;
+	ArtPoint i1, i2;
+	ArtPoint c1, c2;
 
 	re = GNOME_CANVAS_RE (item);
 
 	/* Get canvas pixel coordinates */
 
-	dx = 0.0;
-	dy = 0.0;
-
-	gnome_canvas_item_i2w (item, &dx, &dy);
-
-	gnome_canvas_w2c (item->canvas, dx + re->x1, dy + re->y1, &x1, &y1);
-	gnome_canvas_w2c (item->canvas, dx + re->x2, dy + re->y2, &x2, &y2);
+	gnome_canvas_item_i2w_affine (item, i2w);
+	gnome_canvas_w2c_affine (item->canvas, w2c);
+	art_affine_multiply (i2c, i2w, w2c);
 
+	i1.x = re->x1;
+	i1.y = re->y1;
+	i2.x = re->x2;
+	i2.y = re->y2;
+	art_affine_point (&c1, &i1, i2c);
+	art_affine_point (&c2, &i2, i2c);
+	x1 = c1.x;
+	y1 = c1.y;
+	x2 = c2.x;
+	y2 = c2.y;
+	
 	if (re->fill_set) {
 		if (re->fill_stipple)
 			gnome_canvas_set_stipple_origin (item->canvas, re->fill_gc);
@@ -996,58 +1183,81 @@ gnome_canvas_ellipse_update (GnomeCanvasItem *item, double affine[6], ArtSVP *cl
 {
 	GnomeCanvasRE *re;
 	ArtVpath vpath[N_PTS + N_PTS + 3];
+	ArtVpath *vpath2;
 	double x0, y0, x1, y1;
-	double dx, dy;
 	int i;
 
+#ifdef VERBOSE
+	g_print ("gnome_canvas_rect_update item %x\n", item);
+#endif
+
 	gnome_canvas_re_update_shared (item, affine, clip_path, flags);
 	re = GNOME_CANVAS_RE (item);
 
-	dx = 0.0;
-	dy = 0.0;
+	if (item->canvas->aa) {
 
-	gnome_canvas_item_i2w (item, &dx, &dy);
+#ifdef VERBOSE
+		{
+			char str[128];
 
-	gnome_canvas_w2c_d (item->canvas, dx + re->x1, dy + re->y1, &x0, &y0);
-	gnome_canvas_w2c_d (item->canvas, dx + re->x2, dy + re->y2, &x1, &y1);
-
-	if (re->fill_set) {
-		gnome_canvas_gen_ellipse (vpath, x0, y0, x1, y1);
-		vpath[N_PTS + 1].code = ART_END;
-		vpath[N_PTS + 1].x = 0;
-		vpath[N_PTS + 1].y = 0;
-
-		gnome_canvas_update_svp (item->canvas, &re->fill_svp, art_svp_from_vpath (vpath));
-        } else
-                gnome_canvas_update_svp (item->canvas, &re->fill_svp, NULL);
-
-	if (re->outline_set) {
-		double halfwidth;
-
-		if (re->width_pixels)
-			halfwidth = re->width * 0.5;
-		else
-			halfwidth = re->width * item->canvas->pixels_per_unit * 0.5;
-
-		i = 0;
-		gnome_canvas_gen_ellipse (vpath + i,
-					  x0 - halfwidth, y0 - halfwidth,
-					  x1 + halfwidth, y1 + halfwidth);
-		i = N_PTS + 1;
-		if (x1 - halfwidth > x0 + halfwidth &&
-		    y1 - halfwidth > y0 + halfwidth) {
-			gnome_canvas_gen_ellipse (vpath + i,
-						  x0 + halfwidth, y1 - halfwidth,
-						  x1 - halfwidth, y0 + halfwidth);
-			i += N_PTS + 1;
+			art_affine_to_string (str, affine);
+			g_print ("g_c_r_e affine = %s\n", str);
 		}
-		vpath[i].code = ART_END;
-		vpath[i].x = 0;
-		vpath[i].y = 0;
+		g_print ("item %x (%g, %g) - (%g, %g)\n",
+			 item,
+			 re->x1, re->y1, re->x2, re->y2);
+#endif
 
-		gnome_canvas_update_svp (item->canvas, &re->outline_svp, art_svp_from_vpath (vpath));
-        } else
-                gnome_canvas_update_svp (item->canvas, &re->outline_svp, NULL);
+		if (re->fill_set) {
+			gnome_canvas_gen_ellipse (vpath, re->x1, re->y1, re->x2, re->y2);
+			vpath[N_PTS + 1].code = ART_END;
+			vpath[N_PTS + 1].x = 0;
+			vpath[N_PTS + 1].y = 0;
+
+			vpath2 = art_vpath_affine_transform (vpath, affine);
+
+			gnome_canvas_item_update_svp_clip (item, &re->fill_svp, art_svp_from_vpath (vpath2), clip_path);
+			art_free (vpath2);
+		} else
+			gnome_canvas_item_update_svp (item, &re->fill_svp, NULL);
+
+		if (re->outline_set) {
+			double halfwidth;
+
+			if (re->width_pixels)
+				halfwidth = (re->width / item->canvas->pixels_per_unit) * 0.5;
+			else
+				halfwidth = re->width * 0.5;
+
+			if (halfwidth < 0.25)
+				halfwidth = 0.25;
+
+			i = 0;
+			gnome_canvas_gen_ellipse (vpath + i,
+						  re->x1 - halfwidth, re->y1 - halfwidth,
+						  re->x2 + halfwidth, re->y2 + halfwidth);
+			i = N_PTS + 1;
+			if (re->x2 - halfwidth > re->x1 + halfwidth &&
+			    re->y2 - halfwidth > re->y1 + halfwidth) {
+				gnome_canvas_gen_ellipse (vpath + i,
+							  re->x1 + halfwidth, re->y2 - halfwidth,
+							  re->x2 - halfwidth, re->y1 + halfwidth);
+				i += N_PTS + 1;
+			}
+			vpath[i].code = ART_END;
+			vpath[i].x = 0;
+			vpath[i].y = 0;
+
+			vpath2 = art_vpath_affine_transform (vpath, affine);
+
+			gnome_canvas_item_update_svp_clip (item, &re->outline_svp, art_svp_from_vpath (vpath2), clip_path);
+			art_free (vpath2);
+		} else
+			gnome_canvas_item_update_svp (item, &re->outline_svp, NULL);
+	} else {
+		/* xlib rendering - just update the bbox */
+		get_bounds (re, &x0, &y0, &x1, &y1);
+		gnome_canvas_update_bbox (item, x0, y0, x1, y1);
+	}
 
-	/* FIXME: request repaint */
 }
