@@ -11,7 +11,12 @@
 #include <config.h>
 #include <math.h>
 #include "gnome-canvas-text.h"
+#include <gdk/gdkx.h> /* for BlackPixel */
 
+#include "libart_lgpl/art_affine.h"
+#include "libart_lgpl/art_rgb.h"
+#include "libart_lgpl/art_rgb_bitmap_affine.h"
+#include "gnome-canvas-util.h"
 
 /* This defines a line of text */
 struct line {
@@ -36,6 +41,7 @@ enum {
 	ARG_Y_OFFSET,
 	ARG_FILL_COLOR,
 	ARG_FILL_COLOR_GDK,
+	ARG_FILL_COLOR_RGBA,
 	ARG_FILL_STIPPLE,
 	ARG_TEXT_WIDTH,
 	ARG_TEXT_HEIGHT
@@ -61,6 +67,10 @@ static double gnome_canvas_text_point       (GnomeCanvasItem *item, double x, do
 					     int cx, int cy, GnomeCanvasItem **actual_item);
 static void   gnome_canvas_text_translate   (GnomeCanvasItem *item, double dx, double dy);
 static void   gnome_canvas_text_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
+static void   gnome_canvas_text_render      (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
+
+static GnomeCanvasTextSuckFont * gnome_canvas_suck_font (GdkFont *font);
+static void gnome_canvas_suck_font_free (GnomeCanvasTextSuckFont *suckfont);
 
 
 static GnomeCanvasItemClass *parent_class;
@@ -114,6 +124,7 @@ gnome_canvas_text_class_init (GnomeCanvasTextClass *class)
 	gtk_object_add_arg_type ("GnomeCanvasText::y_offset", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_Y_OFFSET);
 	gtk_object_add_arg_type ("GnomeCanvasText::fill_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_FILL_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasText::fill_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_FILL_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasText::fill_color_rgba", GTK_TYPE_UINT, GTK_ARG_READWRITE, ARG_FILL_COLOR_RGBA);
 	gtk_object_add_arg_type ("GnomeCanvasText::fill_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_FILL_STIPPLE);
 	gtk_object_add_arg_type ("GnomeCanvasText::text_width", GTK_TYPE_DOUBLE, GTK_ARG_READABLE, ARG_TEXT_WIDTH);
 	gtk_object_add_arg_type ("GnomeCanvasText::text_height", GTK_TYPE_DOUBLE, GTK_ARG_READABLE, ARG_TEXT_HEIGHT);
@@ -129,6 +140,7 @@ gnome_canvas_text_class_init (GnomeCanvasTextClass *class)
 	item_class->point = gnome_canvas_text_point;
 	item_class->translate = gnome_canvas_text_translate;
 	item_class->bounds = gnome_canvas_text_bounds;
+	item_class->render = gnome_canvas_text_render;
 }
 
 static void
@@ -171,12 +183,90 @@ gnome_canvas_text_destroy (GtkObject *object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-/* Recalculates the bounding box of the text item.  The bounding box is defined by the text's
- * extents if the clip rectangle is disabled.  If it is enabled, the bounding box is defined by the
- * clip rectangle itself.
- */
 static void
-recalc_bounds (GnomeCanvasText *text)
+get_bounds_item_relative (GnomeCanvasText *text, double *px1, double *py1, double *px2, double *py2)
+{
+	GnomeCanvasItem *item;
+	double x, y;
+	double clip_x, clip_y;
+
+	item = GNOME_CANVAS_ITEM (text);
+
+	x = text->x;
+	y = text->y;
+
+	clip_x = x;
+	clip_y = y;
+
+	/* Calculate text dimensions */
+
+	if (text->text)
+		text->height = (text->font->ascent + text->font->descent) * text->num_lines;
+	else
+		text->height = 0;
+
+	/* Anchor text */
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_SW:
+		break;
+
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_S:
+		x -= text->max_width / 2;
+		clip_x -= text->clip_width / 2;
+		break;
+
+	case GTK_ANCHOR_NE:
+	case GTK_ANCHOR_E:
+	case GTK_ANCHOR_SE:
+		x -= text->max_width;
+		clip_x -= text->clip_width;
+		break;
+	}
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_NE:
+		break;
+
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_E:
+		y -= text->height / 2;
+		clip_y -= text->clip_height / 2;
+		break;
+
+	case GTK_ANCHOR_SW:
+	case GTK_ANCHOR_S:
+	case GTK_ANCHOR_SE:
+		y -= text->height;
+		clip_y -= text->clip_height;
+		break;
+	}
+
+	/* Bounds */
+
+	if (text->clip) {
+		/* maybe do bbox intersection here? */
+		*px1 = clip_x;
+		*py1 = clip_y;
+		*px2 = clip_x + text->clip_width;
+		*py2 = clip_y + text->clip_height;
+	} else {
+		*px1 = x;
+		*py1 = y;
+		*px2 = x + text->max_width;
+		*py2 = y + text->height;
+	}
+}
+
+static void
+get_bounds (GnomeCanvasText *text, double *px1, double *py1, double *px2, double *py2)
 {
 	GnomeCanvasItem *item;
 	double wx, wy;
@@ -250,16 +340,30 @@ recalc_bounds (GnomeCanvasText *text)
 	/* Bounds */
 
 	if (text->clip) {
-		item->x1 = text->clip_cx;
-		item->y1 = text->clip_cy;
-		item->x2 = text->clip_cx + text->clip_cwidth;
-		item->y2 = text->clip_cy + text->clip_cheight;
+		*px1 = text->clip_cx;
+		*py1 = text->clip_cy;
+		*px2 = text->clip_cx + text->clip_cwidth;
+		*py2 = text->clip_cy + text->clip_cheight;
 	} else {
-		item->x1 = text->cx;
-		item->y1 = text->cy;
-		item->x2 = text->cx + text->max_width;
-		item->y2 = text->cy + text->height;
+		*px1 = text->cx;
+		*py1 = text->cy;
+		*px2 = text->cx + text->max_width;
+		*py2 = text->cy + text->height;
 	}
+}
+
+/* Recalculates the bounding box of the text item.  The bounding box is defined by the text's
+ * extents if the clip rectangle is disabled.  If it is enabled, the bounding box is defined by the
+ * clip rectangle itself.
+ */
+static void
+recalc_bounds (GnomeCanvasText *text)
+{
+	GnomeCanvasItem *item;
+
+	item = GNOME_CANVAS_ITEM (text);
+
+	get_bounds (text, &item->x1, &item->y1, &item->x2, &item->y2);
 
 	gnome_canvas_group_child_bounds (GNOME_CANVAS_GROUP (item->parent), item);
 }
@@ -409,6 +513,11 @@ gnome_canvas_text_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 			text->font = gdk_font_load ("fixed");
 			g_assert (text->font != NULL);
 		}
+		if (item->canvas->aa) {
+			if (text->suckfont)
+				gnome_canvas_suck_font_free (text->suckfont);
+			text->suckfont = gnome_canvas_suck_font (text->font);
+		}
 
 		calc_line_widths (text);
 		recalc_bounds (text);
@@ -420,6 +529,11 @@ gnome_canvas_text_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 		text->font = GTK_VALUE_BOXED (*arg);
 		gdk_font_ref (text->font);
+		if (item->canvas->aa) {
+			if (text->suckfont)
+				gnome_canvas_suck_font_free (text->suckfont);
+			text->suckfont = gnome_canvas_suck_font (text->font);
+		}
 		calc_line_widths (text);
 		recalc_bounds (text);
 		break;
@@ -459,17 +573,44 @@ gnome_canvas_text_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		break;
 
 	case ARG_FILL_COLOR:
-		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color))
+		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color)) {
 			text->pixel = color.pixel;
-		else
+			if (item->canvas->aa)
+				text->rgba =
+					((color.red & 0xff00) << 16) |
+					((color.green & 0xff00) << 8) |
+					(color.blue & 0xff00) |
+					0xff;
+			else
+				set_text_gc_foreground (text);
+		} else {
 			text->pixel = 0;
+			if (item->canvas->aa)
+				text->rgba = 0;
+			else
+				set_text_gc_foreground (text);
+		}
 
-		set_text_gc_foreground (text);
 		break;
 
 	case ARG_FILL_COLOR_GDK:
 		text->pixel = ((GdkColor *) GTK_VALUE_BOXED (*arg))->pixel;
-		set_text_gc_foreground (text);
+		if (item->canvas->aa)
+			text->rgba =
+				((color.red & 0xff00) << 16) |
+				((color.green & 0xff00) << 8) |
+				(color.blue & 0xff00) |
+				0xff;
+		else
+			set_text_gc_foreground (text);
+		break;
+
+	case ARG_FILL_COLOR_RGBA:
+		text->rgba = GTK_VALUE_UINT (*arg);
+
+		/* should probably request repaint on the fill_svp */
+		gnome_canvas_item_request_update (item);
+
 		break;
 
 	case ARG_FILL_STIPPLE:
@@ -541,6 +682,10 @@ gnome_canvas_text_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_BOXED (*arg) = color;
 		break;
 
+	case ARG_FILL_COLOR_RGBA:
+		GTK_VALUE_UINT (*arg) = text->rgba;
+		break;
+
 	case ARG_FILL_STIPPLE:
 		GTK_VALUE_BOXED (*arg) = text->stipple;
 		break;
@@ -563,16 +708,35 @@ static void
 gnome_canvas_text_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
 	GnomeCanvasText *text;
+	double x1, y1, x2, y2;
+	ArtDRect i_bbox, c_bbox;
+	int i;
 
 	text = GNOME_CANVAS_TEXT (item);
 
 	if (parent_class->update)
 		(* parent_class->update) (item, affine, clip_path, flags);
 
-	set_text_gc_foreground (text);
-	set_stipple (text, text->stipple, TRUE);
+	if (!item->canvas->aa) {
+		set_text_gc_foreground (text);
+		set_stipple (text, text->stipple, TRUE);
+		get_bounds (text, &x1, &y1, &x2, &y2);
 
-	recalc_bounds (text);
+		gnome_canvas_update_bbox (item, x1, y1, x2, y2);
+	} else {
+		/* aa rendering */
+		for (i = 0; i < 6; i++)
+			text->affine[i] = affine[i];
+		get_bounds_item_relative (text, &i_bbox.x0, &i_bbox.y0, &i_bbox.x1, &i_bbox.y1);
+		art_drect_affine_transform (&c_bbox, &i_bbox, affine);
+#ifdef VERBOSE
+		g_print ("gnome_canvas_text_update (%g, %g) - (%g, %g) -> (%g, %g) - (%g, %g)\n",
+			 i_bbox.x0, i_bbox.y0, i_bbox.x1, i_bbox.y1,
+			 c_bbox.x0, c_bbox.y0, c_bbox.x1, c_bbox.y1);
+#endif
+		gnome_canvas_update_bbox (item, c_bbox.x0, c_bbox.y0, c_bbox.x1, c_bbox.y1);
+
+	}
 }
 
 static void
@@ -587,7 +751,9 @@ gnome_canvas_text_realize (GnomeCanvasItem *item)
 
 	text->gc = gdk_gc_new (item->canvas->layout.bin_window);
 
+#if 0
 	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
+#endif
 }
 
 static void
@@ -603,6 +769,82 @@ gnome_canvas_text_unrealize (GnomeCanvasItem *item)
 		(* parent_class->unrealize) (item);
 }
 
+/* Calculates the x position of the specified line of text, based on the text's justification */
+static double
+get_line_xpos_item_relative (GnomeCanvasText *text, struct line *line)
+{
+	double x;
+
+	x = text->x;
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_SW:
+		break;
+
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_S:
+		x -= text->max_width / 2;
+		break;
+
+	case GTK_ANCHOR_NE:
+	case GTK_ANCHOR_E:
+	case GTK_ANCHOR_SE:
+		x -= text->max_width;
+		break;
+	}
+
+	switch (text->justification) {
+	case GTK_JUSTIFY_RIGHT:
+		x += text->max_width - line->width;
+		break;
+
+	case GTK_JUSTIFY_CENTER:
+		x += (text->max_width - line->width) * 0.5;
+		break;
+
+	default:
+		/* For GTK_JUSTIFY_LEFT, we don't have to do anything.  We do not support
+		 * GTK_JUSTIFY_FILL, yet.
+		 */
+		break;
+	}
+
+	return x;
+}
+
+/* Calculates the y position of the first line of text. */
+static double
+get_line_ypos_item_relative (GnomeCanvasText *text)
+{
+	double x, y;
+
+	y = text->y;
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_NE:
+		break;
+
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_E:
+		y -= text->height / 2;
+		break;
+
+	case GTK_ANCHOR_SW:
+	case GTK_ANCHOR_S:
+	case GTK_ANCHOR_SE:
+		y -= text->height;
+		break;
+	}
+
+	return y;
+}
+
 /* Calculates the x position of the specified line of text, based on the text's justification */
 static int
 get_line_xpos (GnomeCanvasText *text, struct line *line)
@@ -681,6 +923,7 @@ gnome_canvas_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		gdk_gc_set_clip_rectangle (text->gc, NULL);
 }
 
+#ifdef OLD_XFORM
 static double
 gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
 			 int cx, int cy, GnomeCanvasItem **actual_item)
@@ -763,6 +1006,90 @@ gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
 
 	return best / item->canvas->pixels_per_unit;
 }
+#else
+static double
+gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
+			 int cx, int cy, GnomeCanvasItem **actual_item)
+{
+	GnomeCanvasText *text;
+	int i;
+	struct line *lines;
+	int x1, y1, x2, y2;
+	int font_height;
+	int dx, dy;
+	double dist, best;
+
+	text = GNOME_CANVAS_TEXT (item);
+
+	*actual_item = item;
+
+	/* The idea is to build bounding rectangles for each of the lines of text (clipped by the
+	 * clipping rectangle, if it is activated) and see whether the point is inside any of these.
+	 * If it is, we are done.  Otherwise, calculate the distance to the nearest rectangle.
+	 */
+
+	font_height = text->font->ascent + text->font->descent;
+
+	best = 1.0e36;
+
+	lines = text->lines;
+
+	for (i = 0; i < text->num_lines; i++) {
+		/* Compute the coordinates of rectangle for the current line, clipping if appropriate */
+
+		x1 = get_line_xpos (text, lines);
+		y1 = text->cy + i * font_height;
+		x2 = x1 + lines->width;
+		y2 = y1 + font_height;
+
+		if (text->clip) {
+			if (x1 < text->clip_cx)
+				x1 = text->clip_cx;
+
+			if (y1 < text->clip_cy)
+				y1 = text->clip_cy;
+
+			if (x2 > (text->clip_cx + text->clip_width))
+				x2 = text->clip_cx + text->clip_width;
+
+			if (y2 > (text->clip_cy + text->clip_height))
+				y2 = text->clip_cy + text->clip_height;
+
+			if ((x1 >= x2) || (y1 >= y2))
+				continue;
+		}
+
+		/* Calculate distance from point to rectangle */
+
+		if (cx < x1)
+			dx = x1 - cx;
+		else if (cx >= x2)
+			dx = cx - x2 + 1;
+		else
+			dx = 0;
+
+		if (cy < y1)
+			dy = y1 - cy;
+		else if (cy >= y2)
+			dy = cy - y2 + 1;
+		else
+			dy = 0;
+
+		if ((dx == 0) && (dy == 0))
+			return 0.0;
+
+		dist = sqrt (dx * dx + dy * dy);
+		if (dist < best)
+			best = dist;
+
+		/* Next! */
+
+		lines++;
+	}
+
+	return best / item->canvas->pixels_per_unit;
+}
+#endif
 
 static void
 gnome_canvas_text_translate (GnomeCanvasItem *item, double dx, double dy)
@@ -837,3 +1164,173 @@ gnome_canvas_text_bounds (GnomeCanvasItem *item, double *x1, double *y1, double
 	*x2 = *x1 + width;
 	*y2 = *y1 + height;
 }
+
+/* Routines for sucking fonts from the X server */
+
+static GnomeCanvasTextSuckFont *
+gnome_canvas_suck_font (GdkFont *font)
+{
+	GnomeCanvasTextSuckFont *suckfont;
+	int i;
+	int x, y;
+	char text[1];
+	int lbearing, rbearing, ch_width, ascent, descent;
+	GdkPixmap *pixmap;
+	GdkColor black, white;
+	GdkImage *image;
+	GdkGC *gc;
+	guchar *bitmap, *line;
+	int width, height;
+	int black_pixel, pixel;
+
+	suckfont = g_new (GnomeCanvasTextSuckFont, 1);
+
+	height = font->ascent + font->descent;
+	x = 0;
+	for (i = 0; i < 256; i++) {
+		text[0] = i;
+		gdk_text_extents (font, text, 1,
+				  &lbearing, &rbearing, &ch_width, &ascent, &descent);
+		suckfont->chars[i].left_sb = lbearing;
+		suckfont->chars[i].right_sb = ch_width - rbearing;
+		suckfont->chars[i].width = rbearing - lbearing;
+		suckfont->chars[i].ascent = ascent;
+		suckfont->chars[i].descent = descent;
+		suckfont->chars[i].bitmap_offset = x;
+		x += (ch_width + 31) & -32;
+	}
+
+	width = x;
+
+	suckfont->bitmap_width = width;
+	suckfont->bitmap_height = height;
+	suckfont->ascent = font->ascent;
+
+	pixmap = gdk_pixmap_new (NULL, suckfont->bitmap_width,
+				 suckfont->bitmap_height, 1);
+	gc = gdk_gc_new (pixmap);
+	gdk_gc_set_font (gc, font);
+
+	black_pixel = BlackPixel (gdk_display, DefaultScreen (gdk_display));
+	black.pixel = black_pixel;
+	white.pixel = WhitePixel (gdk_display, DefaultScreen (gdk_display));
+	gdk_gc_set_foreground (gc, &white);
+	gdk_draw_rectangle (pixmap, gc, 1, 0, 0, width, height);
+
+	gdk_gc_set_foreground (gc, &black);
+	for (i = 0; i < 256; i++) {
+		text[0] = i;
+		gdk_draw_text (pixmap, font, gc,
+			       suckfont->chars[i].bitmap_offset - suckfont->chars[i].left_sb,
+			       font->ascent,
+			       text, 1);
+	}
+
+	/* The handling of the image leaves me with distinct unease.
+	   But this is more or less copied out of
+	   gimp/app/text_tool.c, so it _ought_ to work. -RLL */
+
+	image = gdk_image_get (pixmap, 0, 0, width, height);
+	suckfont->bitmap = g_malloc0 ((width >> 3) * height);
+
+	line = suckfont->bitmap;
+	for (y = 0; y < height; y++) {
+		for (x = 0; x < width; x++) {
+			pixel = gdk_image_get_pixel (image, x, y);
+			if (pixel == black_pixel)
+				line[x >> 3] |= 128 >> (x & 7);
+		}
+		line += width >> 3;
+	}
+
+	gdk_image_destroy (image);
+
+	/* free the pixmap */
+	gdk_pixmap_unref (pixmap);
+
+	/* free the gc */
+	gdk_gc_destroy (gc);
+
+	return suckfont;
+}
+
+static void
+gnome_canvas_suck_font_free (GnomeCanvasTextSuckFont *suckfont) {
+	g_free (suckfont->bitmap);
+	g_free (suckfont);
+}
+
+static void
+gnome_canvas_text_render      (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
+{
+	GnomeCanvasText *text;
+	guint32 fg_color;
+	double x_start, y_start;
+	double xpos, ypos;
+	struct line *lines;
+	int i, j;
+	double affine[6];
+	GnomeCanvasTextSuckFont *suckfont;
+	int dx, dy;
+	ArtPoint start_i, start_c;
+
+	text = GNOME_CANVAS_TEXT (item);
+	if (!text->text)
+		return;
+
+	suckfont = text->suckfont;
+
+	fg_color = text->rgba;
+
+        gnome_canvas_buf_ensure_buf (buf);
+
+	lines = text->lines;
+	start_i.y = get_line_ypos_item_relative (text);
+
+	art_affine_scale (affine, item->canvas->pixels_per_unit, item->canvas->pixels_per_unit);
+	for (i = 0; i < 6; i++)
+		affine[i] = text->affine[i];
+
+#ifdef VERBOSE
+	{
+		char str[128];
+		art_affine_to_string (str, affine);
+		g_print ("gnome_canvas_text_render %s\n", str);
+	}
+#endif
+	for (i = 0; i < text->num_lines; i++) {
+		if (lines->length != 0) {
+			start_i.x = get_line_xpos_item_relative (text, lines);
+			art_affine_point (&start_c, &start_i, text->affine);
+			xpos = start_c.x;
+			ypos = start_c.y;
+
+			for (j = 0; j < lines->length; j++) {
+				GnomeCanvasTextSuckChar *ch;
+
+				ch = &suckfont->chars[(unsigned char)((lines->text)[j])];
+				affine[4] = xpos;
+				affine[5] = ypos;
+				art_rgb_bitmap_affine (buf->buf,
+						       buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+						       buf->buf_rowstride,
+						       suckfont->bitmap + (ch->bitmap_offset >> 3),
+						       ch->width,
+						       suckfont->bitmap_height,
+						       suckfont->bitmap_width >> 3,
+						       fg_color,
+						       affine,
+						       ART_FILTER_NEAREST, NULL);
+				dx = ch->left_sb + ch->width + ch->right_sb;
+				xpos += dx * affine[0];
+				ypos += dx * affine[1];
+			}
+		}
+
+		dy = text->font->ascent + text->font->descent;
+		start_i.y += dy;
+		lines++;
+	}
+
+	buf->is_bg = 0;
+}
