@@ -1,4 +1,12 @@
 #include <config.h>
+
+/* needed for sigaction and friends under 'gcc -ansi -pedantic' on 
+ * GNU/Linux */
+#ifndef _POSIX_SOURCE
+#  define _POSIX_SOURCE 1
+#endif
+#include <sys/types.h>
+
 #include <unistd.h>
 #include <gnome.h>
 #include <signal.h>
