@@ -21,7 +21,7 @@ xrandom_widgets (GtkTed *t)
 	for (i = 0; i < 4; i++){
 		char buf [40];
 
-		sprintf (buf, "Button-%d", i);
+		g_snprintf (buf, sizeof(buf), "Button-%d", i);
 		l = gtk_button_new_with_label (buf);
 		gtk_widget_show (l);
 		gtk_ted_add (t, l, buf);
@@ -41,7 +41,7 @@ main (int argc, char *argv[])
 {
 	GtkWidget *t, *w;
 	
-	gnome_init ("ted_demo", NULL, argc, argv, 0, NULL);
+	gnome_init ("ted_demo", VERSION, argc, argv);
 
 	gtk_ted_set_app_name ("MyApp");
 	
