@@ -25,7 +25,7 @@ int main(int argc, char *argv[])
 	trig.level = NULL;
 
 	gnomelib_init("trigs", "0.0");
-	fn = g_copy_strings(getenv("srcdir"), "/tests/trigs.in", NULL);
+	fn = g_strconcat(getenv("srcdir"), "/tests/trigs.in", NULL);
 	gnome_triggers_init();
 	gnome_triggers_readfile(fn);
 	gnome_triggers_add_trigger(&trig, "test", "one", NULL);
